/*
 * create_quest.cpp
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "concrete_quests.hpp"
#include "create_quest.hpp"
#include "server_config.hpp"

#include "kconfig/kfile.hpp"
using namespace KConfig;

namespace {
	void PopItemList(ServerConfig &sc, vector<const ItemType *> &output)
	{
		output.clear();
		KFile::List lst(*sc.getKFile(), "list of items");
		for (int i=0; i<lst.getSize(); ++i) {
			lst.push(i);
			const ItemType *it = sc.popItemType();
			if (!it) sc.getKFile()->errExpected("item type");
			output.push_back(it);
		}
	}
}

shared_ptr<Quest> CreateQuest(const string &name, ServerConfig &sc)
{
	shared_ptr<Quest> result;
	if (name == "QuestRetrieve") {
		KFile::List lst(*sc.getKFile(), "", 3, 4);
		lst.push(0);
		int n = sc.getKFile()->popInt();
		lst.push(1);
		vector<const ItemType *> items;
		PopItemList(sc, items);
		lst.push(2);
		string sing = sc.getKFile()->popString();
		lst.push(3);
		string pl = sc.getKFile()->popString(sing + "S");
		result.reset(new QuestRetrieve(n, items, sing, pl));

	} else if (name == "QuestDestroy") {
		vector<const ItemType *> book, wand;
		KFile::List lst(*sc.getKFile(), "", 2);
		lst.push(0);
		PopItemList(sc, book);
		lst.push(1);
		PopItemList(sc, wand);
		result.reset(new QuestDestroy(book, wand));

	} else {
		sc.getKFile()->errExpected("MenuDirective");
	}

	return result;
}
