/*
 * graphic.hpp
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef GRAPHIC_HPP
#define GRAPHIC_HPP

#include "exception_base.hpp"
#include "rgb.hpp"

#include "boost/shared_array.hpp"
using namespace boost;

#include <iosfwd>
#include <string>
using namespace std;

// Graphic is just an incomplete type as far as main program is concerned
class Graphic;

// Create graphics
// hx,hy are "handle" position.
// r,g,b specify transparency colour (0--255). Set r to a negative number for no transparency.
// Throws exception if anything goes wrong.
Graphic * CreateGraphic(shared_array<RGB> pixels, int width, int height, int hx, int hy, int r, int g, int b);

// Load from a BMP resource file
Graphic * CreateGraphicFromFile(const string &filename, int hx, int hy, int r, int g, int b);

// Get properties of graphics
shared_array<RGB> GetPixels(const Graphic &);
int GetWidth(const Graphic &);
int GetHeight(const Graphic &);
int GetHX(const Graphic &);
int GetHY(const Graphic &);
int GetR(const Graphic &);
int GetG(const Graphic &);
int GetB(const Graphic &);
int GetID(const Graphic &);  // each graphic is given a unique ID starting from 1.

// Delete graphics
void DeleteGraphic(Graphic *);

// Serialization [also sets an ID number on the graphic]
// NOTE: Only graphics created with CreateGraphicFromFile can be serialized
void SerializeGraphic(ostream &, Graphic &, int id);
Graphic * CreateGraphicFromStream(istream &);

#endif
